{% extends "layout.html" %}
{% block title %}SlapOS buildout web based runner {% endblock %}
{% block body %}
<div id="home_box">
<div class="inner_box">
<div class="lmenu">
    <h2><a href="{{ url_for('manageRepository')}}#tab1">Clone a Repository into your local Workspace</a></h2>
    <p>Your repository will be cloned into your project folder, you will need the url of the repository.
    This allows you to open and edit your local project copy and synchronize content with online repository.
    </p>
    <img src="{{ url_for('static', filename='images/folder_html.png') }}" />
    <div class="clear"></div>
</div>
<div class="smenu">
    <h2><a href="{{ url_for('openProject', method='open')}}">Open Software Release</a></h2>
    <p>If you have cloned your repository, edit and test your existing Software Release.</p>
    <img src="{{ url_for('static', filename='images/mydocuments.png') }}" />
    <div class="clear"></div>
</div>
<div class="sright_menu">
    <h2><a href="{{ url_for('manageRepository')}}#tab2">Manage Repositories</a></h2>
    <p>You can view details of all repositories and commit your Software Release here.</p>
    <img src="{{ url_for('static', filename='images/manage_repo.png') }}" />
</div>
<div class="clear"></div>
<div class="umenu">
  <h2><a href="{{ url_for('myAccount')}}">Your Account</a></h2>
  <p>Update your account information</p>
  <img src="{{ url_for('static', filename='images/user_card.png') }}" />
</div>
<div class="lmenu smaller">
    <h2><a href="{{ url_for('openProject', method='new')}}">Create your Software Release</a></h2>
    <p>To create a new Software Release, choose the project directory in which you want to create your software.<!-- You will then be able to edit and
    run the new software release.-->
    </p>
    <img src="{{ url_for('static', filename='images/folder_blue.png') }}" />
    <div class="clear"></div>
</div>
<div class="clear"></div>
</div>
</div>
{% endblock %}
